@@ -1,17 +1,23 @@
 package Mac::AETE::Format::Glue;
+use Carp;
 use Data::Dumper;
 use Fcntl;
 use File::Basename;
 use File::Path;
+use File::Spec::Functions qw(catfile);
 use Mac::AETE::Parser;
-use Mac::Glue;
-use MLDBM ('DB_File', $Mac::Glue::SERIALIZER);
+use Mac::Glue::Common;
+use MacPerl;
+use MLDBM ('DB_File', $Mac::Glue::Common::SERIALIZER);
+use Symbol;
 
 use strict;
-use vars qw(@ISA $VERSION);
+use vars qw(@ISA $VERSION $TYPE);
+
+$TYPE = 'McPp';
 
 @ISA = qw(Mac::AETE::Parser);
-$VERSION = '0.30';
+$VERSION = '0.32';
 
 sub fixname {
     (my $ev = shift) =~ s/[^a-zA-Z0-9_]/_/g;
@@ -33,7 +39,7 @@ sub doc_enums {
         $text .= "=item '$n'\n\n=over 4\n\n";
         foreach my $e (keys %{$n{$n}}) {
             $text .= sprintf("=item %s (%s)%s\n\n", $e, $n{$n}{$e}{id},
-                $n{$n}{$e}{desc} ne '' ? ": $n{$n}{$e}{desc}" : '');
+                $n{$n}{$e}{desc} ne '' ? "\n\n$n{$n}{$e}{desc}" : '');
         }
         $text .= "=back\n\n";
     }
@@ -113,7 +119,7 @@ sub doc_classes {
         }
 
         $text .= sprintf("=item %s (%s)%s\n\n", $c, $c{$c}{id},
-            ($d{$c}{desc} ? ": $d{$c}{desc}" : ''));
+            ($d{$c}{desc} ? "\n\n$d{$c}{desc}" : ''));
 
         if (values %p) {
             $text .= "Properties:\n\n";
@@ -132,7 +138,10 @@ sub doc_classes {
         }
 
         if (values %e) {
-            $text .= "Elements: " . join(', ', sort keys %e) . "\n\n";
+            $text .= "Elements:\n\n    " . join(', ', sort
+                map { exists $self->{CLASSNAMES}{$_} ? $self->{CLASSNAMES}{$_} : $_ }
+                map { while (length($_) < 4) { $_ = "$_ " }; $_ }
+                keys %e) . "\n\n";
         }
 
     }
@@ -142,7 +151,7 @@ sub doc_classes {
 }
 
 sub finish {
-    my($self) = @_;
+    my($self, $nopod) = @_;
     my %dbm;
 
     my $path = dirname($self->{OUTPUT});
@@ -151,7 +160,7 @@ sub finish {
 
     unlink $self->{OUTPUT} if $self->{DELETE};
 
-    if (!tie %dbm, 'MLDBM', $self->{OUTPUT}, O_CREAT|O_RDWR|O_EXCL, 0640) {
+    if (!tie %dbm, 'MLDBM', $self->{OUTPUT}, O_CREAT|O_RDWR|O_EXCL, 0644) {
         warn "Can't tie to '$self->{OUTPUT}': $!";
         return;
     }
@@ -160,11 +169,15 @@ sub finish {
     $dbm{CLASS}         = $self->{C};
     $dbm{EVENT}         = $self->{E};
     $dbm{COMPARISON}    = $self->{P};
-    $dbm{ID}            = $self->{ID};
+    $dbm{ID}            = $self->{_parser}{ID};
+    $dbm{APPNAME}       = $self->{_parser}{APPNAME};
+
+    MacPerl::SetFileInfo('McPL', $TYPE, $self->{OUTPUT}) if $^O eq 'MacOS';
+    return 1 if $nopod;
 
     foreach (@{$self}{qw(START FINISH)}) {
         s/__APPNAME__/$self->{TITLE}/g;
-        s/__APPID__/$self->{ID}/g;
+        s/__APPID__/$self->{_parser}{ID}/g;
     }
 
     local *FILE;
@@ -175,14 +188,15 @@ sub finish {
 
     sysopen FILE, $file, O_CREAT|O_WRONLY|O_EXCL
         or die "Can't create file '$file': $!";
-    MacPerl::SetFileInfo(qw(McPL McPp), $self->{OUTPUT});
-    MacPerl::SetFileInfo(qw(uck TEXT), $file);
+    MacPerl::SetFileInfo(qw(uck TEXT), $file) if $^O eq 'MacOS';
 
     print FILE $self->{START};
     print FILE doc_events($self);
     print FILE doc_classes($self);
     print FILE doc_enums($self);
     print FILE $self->{FINISH};
+
+    return 1;
 }
 
 sub new {
@@ -196,7 +210,7 @@ sub new {
 
 sub write_title {
     my($self, $title) = @_;
-    $self->{ID} = (MacPerl::GetFileInfo($title))[0];
+
     $self->{TITLE} = basename($self->{OUTPUT});
 }
 
